/*
  Tool            FontTest
  Author          Carlo Hogeveen
  Website         eCarlo.nl/tse
  Compatibility   Windows GUI TSE v4 upwards
  Version         v0.0.0.2   10 Feb 2025


  THIS TOOL IS STILL IN DEVELOPMENT

  This tool shows a font and its settings in a 16 x 16 character table.
  Both font and settings can be selected.
  The characters are shown as TSE shows them based on TSE settings.
  The TSE session is temporarily shrunk to simplify a side-by-side comparison
  of different fonts and/or settings using two or more TSE sessions.


  GENERAL CONTEXT

  Lots of font-related visual tests are now possible using this tool.
  For example,
  - comparing fonts and their settings is now easier,
  - the impact of settings on characters is now testable,
  - or even something as simple as selecting which font you like best.


  MY CONTEXT

  GUI TSE's default font is Courier New, which is a code page 1252 ("ANSI")
  compatible font.
  I need to know which other fonts are code page 1252 compatible.
  There are references that claim that compatibility, but I also want a way
  to verify these claims by myself.
  This cannot be done automatically. It has to be done visually.
  This tool makes that easier.


  DETAILS

  - Todo:
    Above the table show the actually set font and settings
    instead of the selected ones. (Or both?)

  - As tested with font Courier New:
    If both 3D settings are OFF, then:
    - TSE does not paint the character background color for some characters.
      Because in code page 1252 these are control characters or unused
      characters, this is not a problem for typical editor usage.
    - The same characters have either a half width or a zero width.
      When used in a text, the rest of the line shifts to the left, but the
      cursor is placed as if the character has a normal width.
      This is testable by inserting them with TSE's "ASCII" Chart menu.
      For example, character 14 has half a normal character's width, and
      character 129 has a zero width. The line length counts them both.

  - Fonts' tested code page 1252 ("ANSI") compatibility:
      Compatible:
        Cascadia Code
        Cascadia Mono
        Consolas
        Courier New
        Lucida Console

      Not compatible:
        Courier                 Misses characters 128 - 159 except 146.
        Fixedsys
        Lucida Sans Typewriter  Misses  and .
        Terminal


  TODO
    MUST
    - Add the font size as a setting.
    - Above the table show the actually set font and settings
      instead of the selected font and settings.
    SHOULD
    COULD
    WONT


  HISTORY

  v0.0.0.1    7 Feb 2025
    Inital release.

  v0.0.0.2   10 Feb 2025
  - Fixed: The "Press a key" now always shows on the 1st line.
  - The configured table width is now remembered across TSE sessions.
  - Rewrote the DETAILS section of the documentation.
  - Added new todo points.

*/





// Start of Windows APIs ...

#define WM_SYSCOMMAND 0x0112
#define SC_MAXIMIZE   0xF030

dll "<user32.dll>"
    integer proc SendMessage(integer hwnd,
                             integer msg,
                             integer wparam,
                             integer lparam): "SendMessageA"

    integer proc IsMaximized(integer hwnd)  : "IsZoomed"
end

// End of Windows APIs.





// Global constants and semi-constants
string  MACRO_CFG_SECTION [MAXSTRINGLEN] = ''
integer MACRO_FONT_NAME_HISTORY          = 0
string  MACRO_NAME        [MAXSTRINGLEN] = ''
integer NEW_FONT_COLOR                   = 0


// Global variables
integer new_font_3d_buttons          = 0
integer new_font_3d_chars            = 0
integer new_font_flags               = 0
string  new_font_name [MAXSTRINGLEN] = ''
integer new_font_oem                 = 0
integer new_font_size                = 0
integer new_font_special_effects     = 0
integer new_font_bold                = 0
integer stop_font_menu               = FALSE
integer wide_table                   = FALSE


integer proc write_profile_int(string  section_name,
                               string  item_name,
                               integer item_value)
  integer ok = WriteProfileInt(section_name,
                               item_name,
                               item_value)
  if not ok
    Alarm()
    MsgBox(MACRO_NAME, 'Could not write item "' + item_name +
                       '" to configuration file "tse.ini".')
  endif

  return(ok)
end write_profile_int


/*
  SetFont() always returns 1 in Windows GUI TSE and 0 in the other TSEs.
  This is because Windows returns the closest matching font to the one we
  requested, even if it has to use an alternative font.
  Because Windows never fails, GUI TSE's SetFont() never fails.
  However, here we specifically want the font name we requested, so set_font()
  also returns 0 if the font that Windows set is not the one we requested.
*/
integer proc set_font(string  new_font_name,
                      integer new_font_size,
                      integer new_font_flags)
  string  cur_font_name [MAXSTRINGLEN] = ''
  integer cur_font_size                = 0
  integer cur_font_flags               = 0
  integer n                            = FALSE
  integer result                       = FALSE

  result = SetFont(new_font_name, new_font_size, new_font_flags)

  if result
    GetFont(cur_font_name, cur_font_size, cur_font_flags)

    if cur_font_name <> new_font_name
      result = FALSE
    endif
  endif

  if not result
    n = FindHistoryStr(new_font_name, MACRO_FONT_NAME_HISTORY)
    if n
      DelHistoryStr(MACRO_FONT_NAME_HISTORY, n)
    endif
  endif

  return(result)
end set_font


/*
  bit_values that are truthy or falsy are treated as being 1 and 0
  respectively.
*/
proc set_bitset_bitaddress_bitvalue(var integer bit_set,
                                        integer bit_address,
                                        integer bit_value)
  if bit_value
    bit_set = bit_set | bit_address
  else
    bit_set = ((bit_set ^ MAXINT) | bit_address) ^ MAXINT
  endif
end set_bitset_bitaddress_bitvalue


proc toggle_setting(var integer i)
  i              = not i
  stop_font_menu = FALSE
end toggle_setting


proc select_font_name()
  integer ok                           = TRUE
  integer old_font_flags               = 0
  string  old_font_name [MAXSTRINGLEN] = ''
  integer old_font_size                = 0
  string  tmp_font_name [MAXSTRINGLEN] = ''

  tmp_font_name = new_font_name
  GetFont(old_font_name, old_font_size, old_font_flags)

  if not FindHistoryStr(old_font_name, MACRO_FONT_NAME_HISTORY)
    AddHistoryStr(old_font_name, MACRO_FONT_NAME_HISTORY)
  endif

  if Ask('Select a new font to test:',
         tmp_font_name,
         MACRO_FONT_NAME_HISTORY)
    ok = set_font(tmp_font_name, old_font_size, old_font_flags)
    set_font(old_font_name, old_font_size, old_font_flags)

    if ok
      new_font_name = tmp_font_name
    else
      Warn('Error when setting font "', tmp_font_name, '".')
    endif
  endif

  stop_font_menu = FALSE
end select_font_name


menu font_menu()
  title   = 'Select font and settings, and Go!'
  history = 7

  '&Font'
    [Trim(new_font_name):31],
    select_font_name()

  '&Bold'
    [iif(new_font_bold, 'Yes', 'No'): 3],
    toggle_setting(new_font_bold)

  '&OEM'
    [iif(new_font_oem, 'Yes', 'No'): 3],
    toggle_setting(new_font_oem)

  '&3D Characters'
    [iif(new_font_3d_chars, 'Yes', 'No'): 3],
    toggle_setting(new_font_3d_chars)

  '3&D Buttons'
    [iif(new_font_3d_buttons, 'Yes', 'No'): 3],
    toggle_setting(new_font_3d_buttons)

  '&Wide table'
    [iif(wide_table, 'Yes', 'No'): 3],
    toggle_setting(wide_table)

  '',, _MF_SKIP_

  '                 &Go!'
end font_menu


proc set_window_size()
  if wide_table
    SetVideoRowsCols(25, 33)
  else
    SetVideoRowsCols(25, 18)
  endif
end set_window_size


proc WhenPurged()
  if GetProfileInt(MACRO_CFG_SECTION, 'WideTable', FALSE) <> wide_table
    write_profile_int(MACRO_CFG_SECTION, 'WideTable', wide_table)
  endif
end WhenPurged


proc WhenLoaded()
  MACRO_NAME              = SplitPath(CurrMacroFilename(), _NAME_)
  MACRO_FONT_NAME_HISTORY = GetFreeHistory(MACRO_NAME + ':font_name_history')
  MACRO_CFG_SECTION       = MACRO_NAME + ':Configuration'
  NEW_FONT_COLOR          = Color(Bright Yellow ON Blue)

  wide_table = GetProfileInt(MACRO_CFG_SECTION, 'WideTable', FALSE)
end WhenLoaded


proc Main()
  integer old_font_flags               = 0
  string  old_font_name [MAXSTRINGLEN] = ''
  integer old_font_size                = 0
  integer old_font_special_effects     = Query(SpecialEffects)
  integer old_screen_cols              = Query(ScreenCols)
  integer old_screen_rows              = Query(ScreenRows)
  integer old_show_help_line           = Query(ShowHelpLine)
  integer old_show_main_menu           = Query(ShowMainMenu)
  integer tmp_font_size                = 0
  integer was_maximized                = IsMaximized(GetWinHandle())
  integer x                            = 0
  integer y                            = 0

  GetFont(old_font_name, old_font_size, old_font_flags)

  new_font_3d_buttons      = new_font_special_effects & _USE_3D_BUTTONS_
  new_font_3d_chars        = new_font_special_effects & _USE_3D_CHARS_
  new_font_bold            = new_font_flags           & _FONT_BOLD_
  new_font_flags           = old_font_flags
  new_font_name            = old_font_name
  new_font_oem             = new_font_flags           & _FONT_OEM_
  new_font_size            = old_font_size
  new_font_special_effects = old_font_special_effects

  repeat
    repeat
      stop_font_menu = TRUE
      font_menu()
    until     stop_font_menu
       or not MenuOption()

    if MenuOption()
      set_bitset_bitaddress_bitvalue(new_font_flags,
                                     _FONT_BOLD_,
                                     new_font_bold)
      set_bitset_bitaddress_bitvalue(new_font_flags,
                                     _FONT_OEM_,
                                     new_font_oem)
      set_bitset_bitaddress_bitvalue(new_font_special_effects,
                                     _USE_3D_CHARS_,
                                     new_font_3d_chars)
      set_bitset_bitaddress_bitvalue(new_font_special_effects,
                                     _USE_3D_BUTTONS_,
                                     new_font_3d_buttons)
      Set(SpecialEffects, new_font_special_effects)

      if set_font(new_font_name, new_font_size, new_font_flags)
        Set(ShowHelpLine, OFF)
        Set(ShowMainMenu, OFF)
        UpdateDisplay(_ALL_WINDOWS_REFRESH_)

        tmp_font_size = new_font_size
        set_window_size()
        while Query(ScreenRows) < 25
        and   Query(ScreenRows) >  5
          tmp_font_size = tmp_font_size - 1
          set_font(new_font_name, tmp_font_size, new_font_flags)
          set_window_size()
        endwhile

        KeyPressed()
        UpdateDisplay(_ALL_WINDOWS_REFRESH_)
        KeyPressed()
        Set(Attr, Color(Bright White ON Black))
        ClrScr()

        PutStrAttrXY(1, 3, new_font_name                                          , '', NEW_FONT_COLOR)
        PutStrAttrXY(1, 4, 'Bold:        ' + iif(new_font_bold      , 'Yes', 'No'), '', NEW_FONT_COLOR)
        PutStrAttrXY(1, 5, 'OEM:         ' + iif(new_font_oem       , 'Yes', 'No'), '', NEW_FONT_COLOR)
        PutStrAttrXY(1, 6, '3D Chars:    ' + iif(new_font_3d_chars  , 'Yes', 'No'), '', NEW_FONT_COLOR)
        PutStrAttrXY(1, 7, '3D Buttonss: ' + iif(new_font_3d_buttons, 'Yes', 'No'), '', NEW_FONT_COLOR)

        for y = 0 to 15
          for x = 0 to 15
            if wide_table
              PutStrAttrXY  ((x + 1) * 2    , y + 9, Chr(y * 16 + x), '', NEW_FONT_COLOR)
              if x
                PutStrAttrXY((x + 1) * 2 - 1, y + 9, ' '            , '', NEW_FONT_COLOR)
              endif
            else
              PutStrAttrXY  (x + 2          , y + 9, Chr(y * 16 + x), '', NEW_FONT_COLOR)
            endif
          endfor
        endfor

        Message('Press any key ...')
        GetKey()

        set_font(old_font_name, old_font_size, old_font_flags)
        Set(SpecialEffects, old_font_special_effects)
        Set(ShowHelpLine  , old_show_help_line)
        Set(ShowMainMenu  , old_show_main_menu)
        SetVideoRowsCols(old_screen_rows, old_screen_cols)
        UpdateDisplay(_ALL_WINDOWS_REFRESH_)

        if      was_maximized
        and not IsMaximized(GetWinHandle())
          SendMessage(GetWinHandle(), WM_SYSCOMMAND, SC_MAXIMIZE, 0)
        endif
      endif
    endif
  until not MenuOption()

  PurgeMacro(CurrMacroFilename())
end Main

